<?php

namespace Drupal\selectize\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\OptGroup;
use Drupal\selectize\SelectizeWidgetTrait;

/**
 * Plugin implementation of the 'selectize_widget' widget.
 *
 * @FieldWidget(
 *   id = "selectize_widget",
 *   label = @Translation("Selectize"),
 *   label_singular = @Translation("Selectize"),
 *   label_plural = @Translation("Selects"),
 *   label_count = @PluralTranslation(
 *     singular = @Translation("selectize"),
 *     plural = @Translation("selects"),
 *   ),
 *   field_types = {
 *     "entity_reference",
 *     "list_integer",
 *     "list_float",
 *     "list_string"
 *   },
 *   multiple_values = TRUE
 * )
 */
class SelectizeWidget extends OptionsSelectWidget {

  use SelectizeWidgetTrait;

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();

    $settings = $this->getSettings();
    $settings['maxItems'] = ($cardinality >= 1) ? $cardinality : 999;
    $settings['items'] = $this->getSelectedOptions($items);
    $settings['field_type'] = 'list_string';

    $element = array_merge($element, [
      '#type' => 'selectize',
      '#settings' => $settings,
      '#options' => $this->getOptions($items->getEntity()),
      '#default_value' => $this->getSelectedOptions($items),
      '#multiple' => $this->multiple && count($this->options) > 1,
    ]);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateElement(array $element, FormStateInterface $form_state) {
    if ($element['#required'] && $element['#value'] == '') {
      $form_state->setError($element, t('@name field is required.', ['@name' => $element['#title']]));
    }
    parent::validateElement($element, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getOptions(FieldableEntityInterface $entity) {
    if (!isset($this->options)) {
      // Limit the settable options for the current user account.
      $options = $this->fieldDefinition
        ->getFieldStorageDefinition()
        ->getOptionsProvider($this->column, $entity)
        ->getSettableOptions(\Drupal::currentUser());

      // Add an empty option if the widget needs one.
      if ($empty_label = $this->getEmptyLabel()) {
        $options = ['' => $empty_label] + $options;
      }

      $module_handler = \Drupal::moduleHandler();
      $context = [
        'fieldDefinition' => $this->fieldDefinition,
        'entity' => $entity,
      ];
      $module_handler->alter('options_list', $options, $context);

      array_walk_recursive($options, [$this, 'sanitizeLabel']);

      // Options might be nested ("optgroups"). If the widget does not support
      // nested options, flatten the list.
      if (!$this->supportsGroups()) {
        $options = OptGroup::flattenOptions($options);
      }

      $this->options = $options;
    }
    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  protected function sanitizeLabel(&$label) {
    // Select form inputs allow unencoded HTML entities, but no HTML tags.
    $label = Html::decodeEntities(strip_tags($label));

    // Replace hyphens at the start of an item, example, taxonomy children.
    $label = preg_replace('/^-+/', '', $label);
  }

}
