<?php

namespace Drupal\selectize;

use Drupal\Core\Form\FormStateInterface;

/**
 * Helper methods for the Selectize widgets.
 */
trait SelectizeWidgetTrait {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $delimiter = get_called_class() == 'Drupal\selectize\Plugin\Field\FieldWidget\SelectizeEntityReferenceWidget' ? ';' : ',';
    return [
      'create' => FALSE,
      // If true, when user exits the field (clicks outside of input),
      // a new option is created and selected (if create setting is enabled).
      'createOnBlur' => FALSE,
      'hideTermId' => FALSE,
      // Specifies a RegExp or a string containing a regular expression that
      // the current search filter must match to be allowed to be created. May
      // also be a predicate function that takes the filter text and returns
      // whether it is allowed.
      // string|NULL
      'createFilter' => NULL,
      'delimiter' => $delimiter,
      'delimiter_warning' => FALSE,
      'sortField' => NULL,
      'sortDirection' => 'asc',
      'allowEmptyOption' => FALSE,
      'plugins' => ['remove_button'],
      'highlight' => TRUE,
      'persist' => FALSE,
      'diacritics' => FALSE,
      'closeAfterSelect' => FALSE,
      'placeholder' => 'Start typing to see a list of options...',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $field_name = $this->fieldDefinition->getName();
    $element['plugins'] = [
      '#type' => 'select',
      '#title' => $this->t('Enabled Plugins'),
      '#description' => $this->t('For a demonstration of what these plugins do, visit the <a href="@url">demo site</a>.', ['@url' => 'https://selectize.github.io/selectize.js/']),
      '#default_value' => $this->getSetting('plugins'),
      '#multiple' => TRUE,
      '#options' => $this->pluginsOptions(),
    ];
    $element['sortField'] = [
      '#type' => 'select',
      '#title' => $this->t('Sort field'),
      '#description' => $this->t('Sort Settings'),
      '#default_value' => $this->getSetting('sortField'),
      '#options' => $this->sortOptions(),
    ];
    $element['sortDirection'] = [
      '#type' => 'select',
      '#title' => $this->t('Sort direction'),
      '#description' => $this->t('Direction can be set to "asc" or "desc". The order of the array defines the sort precedence.'),
      '#default_value' => $this->getSetting('sortDirection'),
      '#options' => $this->sortOptions(TRUE),
      '#states' => [
        'visible' => [
          'select[name="fields[' . $field_name . '][settings_edit_form][settings][sortField]"]' => [
            ['value' => 'text'],
            ['value' => 'value'],
          ],
        ],
      ],
    ];
    $element['closeAfterSelect'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Close after selecting an option'),
      '#description' => $this->t('This will close the Selectized display after choosing a value.'),
      '#default_value' => $this->getSetting('closeAfterSelect'),
    ];
    $element['diacritics'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Diacritics Support'),
      '#description' => $this->t('Enable or disable international character support'),
      '#default_value' => $this->getSetting('diacritics'),
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];
    if ($this->getPluginId() == 'selectize_entity_reference_widget' && $this->getSelectionHandlerSetting('auto_create')) {
      $element['createOnBlur'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Create On Blur'),
        '#description' => $this->t('If true, when user exits the field (clicks outside of input), a new option is created and selected (if create setting is enabled)'),
        '#default_value' => $this->getSetting('createOnBlur'),
      ];
      $element['hideTermId'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Hide term ID'),
        '#description' => $this->t('If true, the term ID will be hide for each element. This should be done to prevent possible value duplicates when using autocomplete and manual creation.'),
        '#default_value' => $this->getSetting('hideTermId'),
      ];
      $element['createFilter'] = [
        '#type' => 'textfield',
        '#title' => t('Create filter(REGEX)'),
        '#default_value' => $this->getSetting('createFilter'),
        '#description' => t('Specifies a string containing a regular expression that the current search filter must match to be allowed to be created.'),
      ];
      $element['delimiter'] = [
        '#type' => 'textfield',
        '#title' => t('Delimiter'),
        '#default_value' => $this->getSetting('delimiter'),
        '#description' => t('The string to separate items by. When typing an item in a multi-selection control allowing creation, then the delimiter, the item is added. If you paste delimiter-separated items in such control, the items are added at once.'),
      ];
      $element['delimiter_warning'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Add Delimiter Warning'),
        '#description' => $this->t('If true, add a delimiter warning message to the field description.'),
        '#default_value' => $this->getSetting('delimiter_warning'),
      ];
    }
    else {
      $element['placeholder'] = [
        '#type' => 'textfield',
        '#title' => t('Placeholder'),
        '#default_value' => $this->getSetting('placeholder'),
        '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $settings = $this->getSettings();

    if ($plugins = $settings['plugins']) {
      $plugins = array_intersect_key($this->pluginsOptions(), array_flip($settings['plugins']));
      $summary[] = $this->t('Enabled plugins: @plugins', ['@plugins' => implode(', ', $plugins)]);
    }
    if ($settings['sortField']) {
      $summary[] = $this->t('@sort_field: @sort_direction', [
        '@sort_field' => $this->sortOptions()[$settings['sortField']],
        '@sort_direction' => $this->sortOptions(TRUE)[$settings['sortDirection']],
      ]);
    }
    if ($settings['closeAfterSelect']) {
      $summary[] = $this->t('Closed the Selectized display after choosing a value.');
    }
    if ($settings['diacritics']) {
      $summary[] = $this->t('Enabled international character support.');
    }

    return $summary;
  }

  /**
   * Helper method for getting sort options.
   *
   * @param bool $direction
   *   Indicates to return available sort direction options.
   *
   * @return array
   *   The array of sort options.
   */
  public function sortOptions($direction = FALSE) {
    if ($direction) {
      return [
        'asc' => $this->t('Ascending'),
        'desc' => $this->t('Descending'),
      ];
    }
    else {
      return [
        '' => $this->t('Default sorting'),
        'value' => $this->t('Sort by value'),
        'text' => $this->t('Sort by Text'),
      ];
    }
  }

  /**
   * Helper method for getting available Plugins options.
   *
   * @return array
   *   The array of plugins options.
   */
  public function pluginsOptions() {
    return [
      'remove_button' => $this->t('Remove Button'),
      'drag_drop' => $this->t('Drag & Drop'),
      'restore_on_backspace' => $this->t('Restore on Backspace'),
      'optgroup_columns' => $this->t('Optgroup Columns'),
      'dropdown_header' => $this->t('Dropdown Headers'),
    ];
  }

}
